# -*- coding: utf-8 -*-

#======================================================================
# USAGE: ./tvector_blast.py
# AUTHOR: ZJX (Jim Chu), biojxz@gmail.com
# ORGANIZATION: ECUST
# VERSION: 1.0
# CREATED: 2016年05月29日 16时18分01秒
#======================================================================

"""
开发背景：
Miseq or Hiseq XTen建库中的关键一步——1st(or maybe include 2st)PCR中产物杂质
(fasle-positive)过多，为查明原因，我们目前将PCR产物连接pMD19-T载体后用M13F引物
测序片段的碱基序列。由于测序样品过多及Indx'R' && TcBR-out/SBR-out primer过段的
原因，我们很难用在线ncbi-blast去查明测序片段的成分来源(Indx'R',TcBR-out/SBR-out
 or genome of pichia pastoris).因此，撰写一个可以自动化分析测序结果的脚本很有必
 要。本脚本将成为后续本地化序列比对的基础。
使用说明：
1.将包含测序结果文件(.seq后缀名）的文件夹(download from GWZ email)导入pMD19-T_
exp文件夹(或者含tvector_blast.py & primer.fasta & pichia(db)的其他文件夹；
2.shell中输入python tvector_blast.py <dirname> <word_size = 12> <outfmt = 6>;
"""
import os,re
import pandas as pd
from pyfaidx import Fasta


all_function = {'e':'seq_extraction_merge(tag,result_folder,seq_folder)',
'b':'blastn(tag,result_folder,subtype,dbseq,seq,task,outfmt,word_size)',
'i':'internal_seq_extract_and_blastn(tag,result_folder,subtype,dbseq,seq,task,outfmt,word_size)'}

#tag表示为本次测序文件的标签，由测序文件夹名决定，所以必要时可修改文件夹名
def seq_extraction_merge(tag,result_folder,seq_folder):
    if '.zip' in seq_folder:
        os.system('unzip -q '+seq_folder)
        seq_folder = seq_folder[:-4]
    if not os.path.exists(result_folder):
        os.mkdir(result_folder)
    if not os.path.exists('%s/%s.seq'%(result_folder,tag)):
        os.system("cat %s/*.seq > %s/%s.seq"%(seq_folder,result_folder,tag))

#seq contains short sequences, primer sequences and so on
def blastn(tag,result_folder,subtype,dbseq,seq,task,outfmt,word_size): 
    if not os.path.exists(result_folder):
        os.mkdir(result_folder)
    CMD = "blastn -%s %s -query %s -task %s -word_size %d -outfmt %d -out %s/%s.\
blast%d.xls"%(subtype,dbseq,seq,task,word_size,outfmt,result_folder,tag,outfmt)
    os.system(CMD)
    return CMD
    
def internal_seq_extract_and_blastn(tag,result_folder,subtype,dbseq,seq,task,outfmt,word_size):
    df = pd.read_table("%s/%s.blast6.xls"%(result_folder,tag),header=None)
    #count align_reads of same sample
    gb1 = pd.DataFrame(df.groupby(1).size())
    #gb1 = gb1.drop(1,axis=0)
    #extract min site of internal seq
    gb2 = pd.DataFrame(df.groupby(1)[9].min())
    #extract max site of internal seq
    gb3 = pd.DataFrame(df.groupby(1)[9].max())
    #merge gb1,gb2 and gb3
    df_9merge = gb1.join(gb2)
    df_9merge = pd.merge(df_9merge,gb3,left_index=True,right_index=True,how="left")
    df_9merge.to_csv('%s/%s_Pp.site.xls'%(result_folder,tag),sep='\t',header=False)
    internal_seqf = '%s/%s_Pp.seq'%(result_folder,tag)
    fout = open(internal_seqf,'w')
    Tvector_seq_name = result_folder + '/' + tag + '.seq'
    Tvector_seq = Fasta(Tvector_seq_name)
    with open('%s/%s_Pp.site.xls'%(result_folder,tag)) as f:
        for i in f:
            i = i.split('\t')
            start,end = int(i[2]),int(i[3])
            if start != end:
                # start and end seq of internal(Pp)
                internal_seq = str(Tvector_seq[i[0]][start:end-1])  
                print >>fout,'>'+i[0]+'_internal\n'+internal_seq
        fout.close()   
    CMD = blastn(tag+'_Pp',result_folder,'db','pichia',internal_seqf,'blastn',outfmt,word_size+6)
    #info = "Internal seq blastn Pp is over! The blastn command is '%s'"%CMD
    return CMD
    

if __name__ == "__main__":
    import log,sys
    logger = log.createCustomLogger('root')
    dirname = sys.argv[1]
    tag = re.split("_|/.",dirname)[0]
    
    logger.info("Note: All data results will be in %s_result folder"%tag)
    logger.info("primer.seq blastn %s.seq is begining..."%tag)
    blastn(dirname,tag,'4')
    CMD1 = blastn(dirname,tag,'6')
    logger.info("pr blastn is over!\nThe blastn command is '%s'"%CMD1)
    
    logger.info("%s_Pp.seq is being extracted and blastn pichia.db..."%tag)
    internal_seq_extract_and_blastn(tag,'4')
    CMD2 = internal_seq_extract_and_blastn(tag,'6')
    logger.info("Pp blastn is over!\nThe blastn command is '%s'"%CMD2)
    
    


#! /usr/local/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function
import os
import sys
import pickle
import shutil
import numpy
import pandas as pd
from Bio.Seq import Seq

#毕赤酵母基因组信息更新    
def chr_annot_update():
    from Bio import Entrez
    if not os.path.exists('chr_annot'):
        #shutil.rmtree('chr_annot')
        os.mkdir('chr_annot')
    Entrez.email="biojxz@163.com"
    Chr={'chr1_annot':'238029138','chr2_annot':'238030677',\
    'chr3_annot':'238032011','chr4_annot':'238033210'}
    for key in Chr:
        f=open('%s.xls'%key,'w')
        handle = Entrez.efetch(db="nucleotide", id=Chr[key],rettype="ft",retmode="text")
        f.write(handle.read())
        f.close()
        shutil.move('%s.xls'%key,'chr_annot')

#毕赤酵母染色体信息格式修改
class Chr_annot_mod():
    def __init__(self):
        """Chr_annot_mod:please input the directory of chromosome annotation \
        files in file_deal function,the default directory is 'chr_annot'
        """ 
    
    def file_deal(self,annot_dir='chr_annot'):
        for i,j,k in os.walk(annot_dir):
            print("\nModifying files as follow:")
            for n in k:
                if '_m' not in n:
                    print(n)
                    self.modify('%s/%s'%(i,n))               
    
    def modify(self,annot_file):    
        f1=open('%s'%annot_file)
        f2=open('%s_m.pkl'%annot_file.split(".")[0],'wb')            
        gene,CDS,annot,Lcds,n={},{},{},[],0       
        #List=[['IGR_s1','IGR_s2','gene_logo','gene_s1','gene_s2','Ori','CDS_s1',\
        #'CDS_s2','Intron_num','CDS_len','Intron_s1','Intron_s2','annot']]
        List=[]
        for i in f1:
            i=i.rstrip()
            i=i.split('\t')
            if i[0] != '':
                try:
                    if i[2]=='gene':
                        CDS[n]=Lcds
                        n=n+1
                        gene[n]=i[4],i[0],i[1]
                        Lcds=[]
                    else:
                        Lcds.append([i[0],i[1]])
                        try:
                            if i[3]=='product':
                                annot[n]=i[4]  #annotation
                        except:
                            pass                               
                except:
                    Lcds.append([i[0],i[1]])
        CDS[n]=Lcds
        End_b,CDS_e_b=0,0
        for i in range(1,n+1):
            count=0
            Start,End=int(gene[i][1]),int(gene[i][2])
            for j in CDS[i]:        
                CDS_s,CDS_e=int(j[0]),int(j[1])
                length=CDS_e-CDS_s    
                if End>Start:
                    length2=End-Start
                    if count>0:
                        List.append([End_b+1,Start-1,gene[i][0],Start,End,'+',\
                        CDS_s,CDS_e,length2+1,length+1,count,CDS_e_b+1,CDS_s-1,annot[i]]) 
                    else:      
                        List.append([End_b+1,Start-1,gene[i][0],Start,End,'+',\
                        CDS_s,CDS_e,length2+1,length+1,count,0,0,annot[i]]) 
                    End_b,CDS_e_b=End,CDS_e
                else:
                    length2=Start-End
                    if count>0:
                        List.append([End_b+1,End-1,gene[i][0],End,Start,'-',\
                        CDS_e,CDS_s,length2+1,-length+1,count,CDS_e_b+1,CDS_e-1,annot[i]])
                    else:
                        List.append([End_b+1,End-1,gene[i][0],End,Start,'-',\
                        CDS_e,CDS_s,length2+1,-length+1,count,0,0,annot[i]])   #,annot[i]
                    End_b,CDS_e_b=Start,CDS_s
                count=count+1
        pickle.dump(List,f2)
        f2.close()

#截短reads读长
def cutreads(Tndata,length):
    f1 = open('2.Tndata_raw/%s_cr'%Tndata,'w')
    f2 = open('2.Tndata_raw/'+Tndata)
    count = 0
    for i in f2:
        i = i.rstrip()
        count += 1
        line = i if count%2 else i[:length]
        print(line,file=f1)
    f1.close()
    f2.close()

#去除reads两端的接头
def cutadapt(Tndata,transposon,tag,pe):
    ADAPTER_FWD = transposon
    ADAPTER_REV = str(Seq(ADAPTER_FWD).reverse_complement())
    ADAPTER_LNK = 'ATACCACGAC'
    ADAPTER_LNK_R = 'GTCGTGGTAT'
    pm_t="-O 17 -e 0.2  --match-read-wildcards --discard-untrimmed -f fastq" 
    pm_l="-O 5 -e 0.1 -m 10 -f fastq"
    if pe:
        Tndata1 = Tndata
        Tndata2 = Tndata.replace("_R1","_R2")
        tag1 = tag
        tag2 = tag.replace("_R1","_R2")
        adapter_rev_p,adapter_link_p = ' -A' + ADAPTER_REV,' -G' + ADAPTER_LNK_R
        os.system('cutadapt -g %s -A %s %s -o 3.Cutadapt_trimmed/%s_t -p 3.Cutadapt_trimmed/%s_t 2.Tndata_raw/%s 2.Tndata_raw/%s'\
        %(ADAPTER_FWD,ADAPTER_REV,pm_t,tag1,tag2,Tndata1,Tndata2))
        os.system('cutadapt -a %s -G %s %s -o 3.Cutadapt_trimmed/%s_t_l -p 3.Cutadapt_trimmed/%s_t_l 3.Cutadapt_trimmed/%s_t 3.Cutadapt_trimmed/%s_t'\
        %(ADAPTER_LNK,ADAPTER_LNK_R,pm_l,tag1,tag2,tag1,tag2))
    else:
        adapter_rev_p,adapter_link_p = '',''
        if tag + '_t' not in os.listdir('3.Cutadapt_trimmed'):
            os.system('cutadapt -g %s %s -o 3.Cutadapt_trimmed/%s_t 2.Tndata_raw/%s'%(ADAPTER_FWD,pm_t,tag,Tndata))
        #cutadapt -a ATACCACGAC -O 5 -e 0.1 -m 10 -f fastq -o 3.Cutadapt_trimmed/tss4_t_l 3.Cutadapt_trimmed/tss4_t
        os.system('cutadapt -a %s %s -o 3.Cutadapt_trimmed/%s_t_l 3.Cutadapt_trimmed/%s_t'%(ADAPTER_LNK,pm_l,tag,tag))
    print("\n========== Cutadapt finished for: %s ==========\n\n"%Tndata) 

#比对毕赤酵母基因组
def bowtie(tag_cutadapt,tag_bowtie):   
    os.system("bowtie -p 16 -v 3 -a --best --strata -m 1 -q pichia/genome \
    3.Cutadapt_trimmed/%s 4.Bowtie_data_dir/%s"%(tag_cutadapt,tag_bowtie))
    #shutil.move(tag_bowtie,'4.Bowtie_data_dir/')
    print("\n========== Bowtie finished for: %s ==========\n\n"%tag_cutadapt)

#去除reads匹配到基因组上相同位点的重复
def unique(file_impt,tag,transposon):
    if not os.path.exists(tag):
        os.mkdir(tag)
    L=['NC_012963.1','NC_012964.1','NC_012965.1','NC_012966.1']

    df=pd.read_table("%s"%file_impt,header=None) 
    #drop the length of site base is less 20bp
    df=df[df[4].map(lambda x:len(x)>=20)].iloc[:,[2,3,1,4]] ##reads读长的限制，gonna to cancell in future
    #site amendment
    if 'TcB' in transposon:
         df['site_amdt'] = numpy.where(df[1]=='+',df[3]+4,df[3]+df[4].str.len()-4)
    if 'SB' in transposon:
        df['site_amdt'] = numpy.where(df[1]=='+',df[3]+1,df[3]+df[4].str.len()-1)
    gb1=df.groupby([2,'site_amdt',4]).count()
    gb2=df.groupby([2,'site_amdt']).count()
    df['sn']=df[4].str.len()
    df=pd.merge(df,gb1,left_on=[2,'site_amdt',4],right_index=True,how='left')

    def dereplicate1(df,n=1,columns1='3_y',columns2='sn'):
        #this sort will miss long length of site base
        return df.sort_values(by=[columns1,columns2])[-n:]
    df=df.groupby([2,'site_amdt'],group_keys=False).apply(dereplicate1)
    df=pd.merge(df,gb2,left_on=[2,'site_amdt'],right_index=True,how='left')\
    .iloc[:,[0,4,2,3,6,8]]

    def dereplicate2(s,n,transposon):  #mark similar reads
        Ls=df[df[2].isin([s])].values
        f1=open("%s/%s_chr%d_alldp.xls"%(tag,tag,n),'w')
        a0,e0,Ta0,D1=0,0,"SS",{}
        for i in Ls:
            [a,c,b,d,e]=i[1:]
            if transposon == 'SB':
                Ta=b[:2] if c=='+' else b[-2:]
            if transposon == 'TcBR' or transposon == 'TcBL':
                Ta=b[3:5] if c=='+' else b[-5:-3]
            if a>a0+3:
                D1[a]='Unique_read'
                a0,c0,e0,Ta0=a,c,e,Ta
            else:
                if c==c0:
                    if Ta=='TA' and Ta0=='TA':
                        if e>e0:
                            if a==a0+1:
                                D1[a],D1[a0]='SimTRemain','Similar'
                            else:
                                D1[a],D1[a0]='SimTRemain','SimT'
                            a0,c0,e0,Ta0=a,c,e,Ta
                        else:
                            if a==a0+1:
                                D1[a],D1[a0]='Similar','SimTRemain'
                            else:                            
                                D1[a],D1[a0]='SimT','SimTRemain'  
                    elif Ta=='TA' and Ta0!='TA':
                        D1[a],D1[a0]='Remain','Similar'
                        a0,c0,e0,Ta0=a,c,e,Ta
                    elif Ta!='TA' and Ta0=='TA':
                        D1[a0],D1[a]='Remain','Similar'
                    else:
                        if e>e0:
                            D1[a],D1[a0]='SimFRemian','Similar'
                            a0,c0,e0,Ta0=a,c,e,Ta
                        else:
                            D1[a0],D1[a]='SimFRemian','Similar'
                else:
                    D1[a]='Unique_read'
                    a0,c0,e0,Ta0=a,c,e,Ta
        before_unique=0
        for i in Ls:
            before_unique=before_unique+1
            if D1[i[1]] not in ['Similar','SimT']:
                f1.write("chr%d\t%d\t%s\t%s\t%d\t%d\t%s\n"\
                %(n,i[1],i[2],i[3],i[4],i[5],D1[i[1]]))
        f1.close()
        os.system("cat %s/*chr*_alldp.xls > %s/%s_alldp.xls"%(tag,tag,tag))
        return before_unique
    D2={}
    for i in range(4):
        d2=dereplicate2(L[i],i+1,transposon)
        D2['befuniq_chr'+str(i+1)]=d2
    return D2

#插入位点分类及修饰处理                                              
def site_annot(argv1,argv2):
    f=open('chr_annot/%s_annot_m.pkl'%argv2,'rb')
    List=pickle.load(f)
    f1=open('%s/%s_%s_alldp.xls'%(argv1,argv1,argv2))
    f2=open('%s/%s_%s_IGS.xls'%(argv1,argv1,argv2),'w')
    f3=open('%s/%s_%s_CDS.xls'%(argv1,argv1,argv2),'w')
    f4=open('%s/%s_%s_Intron.xls'%(argv1,argv1,argv2),'w')
    All_count=IGS_count=CDS_count=Intron_count=0
    List_tail={'chr1':2796884,'chr2':2388753,'chr3':2243457,'chr4':1774176}
    for i in f1:
        All_count=All_count+1
        i=i.split()
        i[1]=int(i[1])
        if i[1]<=List_tail[argv2]:
            for j in List:
                if j[3]>i[1]:
                    IGS_count=IGS_count+1
                    f2.write('%s\t%d\t%s\tIGS\t%d\t%d\t%s\t%d\t%d\t%s\t%d\t%d\
                    \t%d\t%d\t%d\t%d\t%d\t%s\n'%(i[0],i[1],i[2],j[0],j[1],j[2],j[3]\
                    ,j[4],j[5],j[6],j[7],j[8],j[9],j[10],j[11],j[12],j[13]))
                    break
                elif j[4]>=i[1]:
                    if i[1]>=j[6] and i[1]<=j[7]:
                        CDS_count=CDS_count+1
                        f3.write('%s\t%d\t%s\tCDS\t%d\t%d\t%s\t%d\t%d\t%s\t%d\t%d\
                        \t%d\t%d\t%d\t%d\t%d\t%s\n'%(i[0],i[1],i[2],j[0],j[1],j[2],j[3]\
                        ,j[4],j[5],j[6],j[7],j[8],j[9],j[10],j[11],j[12],j[13]))
                        break
                    elif i[1]>=j[10] and i[1]<=j[11]:
                        Intron_count=Intron_count+1
                        f4.write('%s\t%d\t%s\tIntron\t%d\t%d\t%s\t%d\t%d\t%s\t%d\t%d\
                        \t%d\t%d\t%d\t%d\t%d\t%s\n'%(i[0],i[1],i[2],j[0],j[1],j[2],j[3]\
                        ,j[4],j[5],j[6],j[7],j[8],j[9],j[10],j[11],j[12],j[13]))
                        break
        else:
            f2.write('%s\t%d\t%s\tIGS\t%d\t%d\t%s\t%d\t%d\t%s\t%d\t%d\
            \t%d\t%d\t%d\t%d\t%d\t%s\n'%(i[0],i[1],i[2],j[0],j[1],j[2],j[3]\
            ,j[4],j[5],j[6],j[7],j[8],j[9],j[10],j[11],j[12],j[13]))
            IGS_count=IGS_count+1
    print('-----The number of All insertion reads of %s is %d-----'%(argv2,All_count))
    print('-----The number of CDS insertion reads of %s is %d-----\n'%(argv2,CDS_count))
    f.close()
    f1.close()
    f2.close()
    f3.close()
    f4.close()
    return {'All_count':All_count,'IGS_count':IGS_count,'CDS_count':CDS_count,'Intron_count':Intron_count}

#基因内部插入个数及位置
def gene_insert(dirname,chro):
    #df1 = pd.read_table('../chr_annot/chr1_CDS.xls',header=None)
    df1 = pd.read_table('chr_annot/%s_CDS.xls'%chro,header=None)
    #df2 = pd.read_table('T16728-5aTb_combined_R1_chr1_CDS.xls',header=None)
    df2 = pd.read_table('%s/%s_%s_CDS.xls'%(dirname,dirname,chro),header=None)
    df2 = df2.iloc[:,[6,1,7,12]]
    #统计每个基因的插入个数
    gb = df2.groupby(6).size()
    #gb为series格式，需数据框化
    dgb1 = pd.DataFrame(gb)
    df2[13] = (df2[1]-df2[7])/df2[12]
    dgb2 = df2.iloc[:,[0,4]].groupby(6).sum()
    dgb3 = df2.iloc[:,[0,4]].groupby(6).min()
    #outer表示并集
    dgb4 = pd.merge(dgb2,dgb3,left_index=True,right_index=True,how='outer')
    df3 = pd.merge(dgb4,dgb1,left_index=True,right_index=True,how='outer')
    df3['site_ratio'] = df3['13_x']/df3[0]
    df4 = pd.merge(df1,df3,left_on=0,right_index=True,how='outer')
    df4 = df4.iloc[:,[0,7,8,6,2,3,4]].fillna(0)
    ##目前暂定义最小插入位点大于等于0.7的为必须基因
    df4['estag'] = numpy.where((df4['13_y'] > 0) & (df4['13_y'] < 0.7) & (df4['0_y'] > 0),'Non-essential','Essential')
    df4.iloc[:,[0,7,1,2,3,4,5,6]].to_csv('%s/%s_%s_CDS_insert.xls'%(dirname,dirname,chro),sep='\t',header=False,index=False)

def mapes(estype,dirname):  #df1 = pd.read_table('essential_gene/ESpredictbySL.txt',)
    df1 = pd.read_table('essential_gene/%s.txt'%estype,header=None)
    df2 = pd.read_table('%s/%s_CDS_insert.xls'%(dirname,dirname),header=None)
    df = pd.merge(df1,df2,left_on=0,right_on=0,how='right')
    df.to_csv('%s/%s_CDS_insert_es.xls'%(dirname,dirname),sep='\t',header=False,index=False)


"""
log.py
=====

Setup logging utils for nested module logging

Adapted from the accepted answer here: http://stackoverflow.com/questions/7621897/python-logging-module-globally
"""

import logging


def createCustomLogger(name):
    formatter = logging.Formatter(fmt='[%(asctime)s][%(levelname)s][%(module)s] %(message)s', datefmt='%m/%d %I:%M:%S%p')

    handler = logging.StreamHandler()
    handler.setFormatter(formatter)

    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    logger.addHandler(handler)
    return logger




from __future__ import print_function
import tnseqdata_visualization as tv

all_function = {'w':'weblogo(dirname)','s':'site_distribution(dirname)','d':'dnaplotter_artemis(dirname)'}
def weblogo(dirname):
    if '/' in dirname:
        dirname = dirname[:-1]
    tv.Weblogo(dirname)

def site_distribution(dirname):
    if dirname[-1] == '/':
        dirname = dirname[:-1]
    for i in range(1,5):
        chro = 'chr%d'%i
        tv.Site_distribution(dirname,chro)
    tv.R_sd(dirname)

def dnaplotter_artemis(dirname):
    if dirname[-1] == '/':
        dirname = dirname[:-1]
    tv.dnaplotter_artemis(dirname)



# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------------#
# USAGE: ./jx                                                                        #
# AUTHOR: ZJX (Jim Chu), biojxz@gmail.com                                            #
# ORGANIZATION: ECUST                                                                #
# VERSION: 1.0                                                                       #
# CREATED: 2016年08月23日 15时20分01秒                                               #
#------------------------------------------------------------------------------------#

from __future__ import print_function
import os
import sys
import numpy
import pandas as pd

all_function = {'g':'gene_insert_density(filename,method,all_insert)','e':'es_map(filename,estypefile)'}

#分析指定文件夹内'*_CDS.xls'类文件的基因内部插入个数及位置
#参考Henry L. Levin的方法，必须基因非必须基因的分界线：20 inserts/kb/million
def gene_insert_density(filename,method,all_insert):
#'gene_length'可能含intron,eg:df = pd.read_table('1a2l8Tb21l22ab/T16728-1a_combined_R1_CDS.xls',usecols=[0,1,6,7,12],names=['chr','insert_site','gene_id','gene_start','gene_length'])
    df = pd.read_table(filename,usecols=[0,1,6,7,12],names=['chr','insert_site','gene_id','gene_start','gene_length'])
    df_annot = pd.read_table('/home/user/eib202epgs/zjx/tnseq/chr_annot/All_CDS.xls',names=['chr','gene_id','gene_length','gene_direct','gene_annot']) #暂时去除基因的方向
    #统计每个基因的插入个数
    if method == 2:
        df = df.groupby(['chr','insert_site'],as_index=False).first()
    gb_size = df.groupby(['chr','gene_id']).size()
    #gb为series格式，需数据框化
    dgb_size = pd.DataFrame(gb_size)
    #计算插入位点在基因ORF的位置，比如在中间则表示为0.5（称之为位点率）
    df['insert_ratio'] = (df['insert_site']-df['gene_start'])/df['gene_length']
    #计算某基因所以插入位点率的综合
    #dgb = df.groupby(['chr','gene_id','gene_length','annot']).count()['insert_site']
    dgb_sum = df.loc[:,['gene_id','insert_ratio']].groupby('gene_id').sum()
    #计算某基因所以插入位点率的最小值
    dgb_min = df.loc[:,['gene_id','insert_ratio']].groupby('gene_id').min()
    #outer表示并集
    dgb_sum_min = pd.merge(dgb_sum,dgb_min,left_index=True,right_index=True,how='outer')
    dgb_merge = pd.merge(dgb_sum_min,dgb_size,left_index=True,right_index=True,how='outer')
    dgb_merge['insert_ratio_x_ave'] = dgb_merge['insert_ratio_x']/dgb_merge[0]
    #基因长度，注释等信息重注释（回归）
    df_merge = pd.merge(df,dgb_merge,left_on=['chr','gene_id'],right_index=True,how='left')
    df_edition1 = df_merge.iloc[:,[0,1,2,4,8,5,7,9]]
    df_edition1.to_csv('%s_edition1.xls'%filename.split('.')[0],sep='\t',index=False,header=['chr','insert_site','gene_id','gene_length','insert_num','insert_ratio','insert_ratio_min','insert_ratio_ave'])
    df_edition1_mod1 = df_edition1.groupby(['chr','gene_id'],as_index=False).first()
    #删除某一列，用drop
    df_edition1_mod2 = df_edition1_mod1.drop(['insert_site','gene_length','insert_ratio'],1)
    df_merge2 = pd.merge(df_annot,df_edition1_mod2,left_on=['chr','gene_id'],right_on=['chr','gene_id'],how='left')
    df_final = df_merge2.fillna(0)
    ##计算所有独立插入
    all_insert_actually = all_insert if all_insert else df_final[0].sum()
    df_final['insert_freq'] = numpy.where(df_final['insert_ratio_y'] < 0.9,(1000000*1000*df_final[0])/(all_insert_actually*df_final['gene_length']),0)
    df_final.iloc[:,[0,1,2,3,5,6,7,8,4]].to_csv('%s_edition2.xls'%filename.split('.')[0],sep='\t',index=False,header=['chr','gene_id','gene_length','gene_direct','insert_num','insert_ratio_min','insert_ratio_ave','insert_freq','gene_annot'])

def esmap_fig(tag1,tag2):
    f=open('%s_%s_esfig_fig.R'%(tag1,tag2),'w')
    Rscript="""library("ggplot2")
    fa<-read.table("%s_%s_esfig.xls")
    colnames(fa) <-c("estype","insert_freq")
    fb<-fa[fa[,2]<600,]
    pdf("%s_%s_esfig.pdf")
    qplot(insert_freq,data=fb,geom = "density",colour=estype)
    dev.off()
    """%(tag1,tag2,tag1,tag2)
    print(Rscript,file=f)
    f.close()
    os.system('R2 CMD BATCH %s_%s_esfig_fig.R'%(tag1,tag2))

def es_map(filename,estypefile):  #df1 = pd.read_table('essential_gene/ESpredictbySL.txt',)
    tag1 = filename.split('.')[0]
    tag2 = estypefile.split('.')[0]
    df1 = pd.read_table('/home/user/eib202epgs/zjx/tnseq/7.essential_analysis/Material/%s'%estypefile)
    df2 = pd.read_table('%s_edition2.xls'%tag1)
    df = pd.merge(df1,df2,left_on='gene_id',right_on='gene_id',how='right')
    df_mod = df.iloc[:,[2,0,1,8,3,4,5,6,7,9]]
    ##去掉未匹配的数据
    df_mod2 = df_mod.dropna()
    df_mod2['estype'] = numpy.where(df_mod2['mapES'].str[-2:]=='ES','Essential','Nonessential')
    df.to_csv('%s_%s_esdata.xls'%(tag1,tag2),sep='\t',index=False)
    df_mod2.to_csv('%s_%s_esfig.xls'%(tag1,tag2),sep='\t',index=False,columns=['estype','insert_freq'],header=False)
    esmap_fig(tag1,tag2)







# -*- coding: utf-8 -*-
#libreadcounts1_vs_libreadcounts2
#warning:输入的库文件名filename的开头必须是转座子名称的第一个字母且大写！！

"""
模块功能说明：
1. Readcounts_diff = (Readcounts1 – Readcounts2)/Max(Readcounts1,Readcounts2)
2. 我们定义，两个library的测序reads数差异程度用参数Readcounts_diff表示；
3. 本脚本的作用是比较两个library的测序reads数差异。如某位点仅在lib1中有reads，则Readcounts_diff = 1;
反之,如某位点仅在lib2中有reads,则Readcounts_diff = -1;两个lib都测出reads时-1< Readcounts_diff <1
4. 将两个lib测出的所有sites按染色体位点从小到大进行排序，以排序的序号为横坐标，Readcounts_diff为纵坐标，
绘制散点图(位点插入reads数测序差异图)
5. 位点插入reads数测序差异图可反映两个库的重叠性，进而可在一定程度上判断库的饱和性，转座子随机性和互补性
"""

from __future__ import print_function
import re
import os,sys
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

##列出所有函数
all_function={'l':'libreadcounts1_vs_libreadcounts2(filename1,filename2)','p':'libs_plus(filenames)'}

def libreadcounts1_vs_libreadcounts2(filename1,filename2):
    #判断lib-filename是否合法
    if (filename1[0] not in 'TS') and (filename2[0] not in 'TS'):
        print("Usage:filename's first letter must be 'S' or 'T'!")   
    pwd=os.getcwd() 

    #以符号'_'或'.'为分隔符，注意点号需要转义
    tag1=re.split("_|\.",filename1)[0]
    tag2=re.split("_|\.",filename2)[0] 
    
    #使用pandas.read_table打开两个要vs(pk)的两个文件
    df1=pd.read_table('%s/%s'%(pwd,filename1),names=['chr','site','orientation','seq','count1','count2','rep'])
    df2=pd.read_table('%s/%s'%(pwd,filename2),names=['chr','site','orientation','seq','count1','count2','rep'])

    #合并两个数据框，同时具有去重复的作用，'outer'表示为并集
    df_merge=pd.merge(df1,df2,on=['chr','site'],how='outer')
    df_merge=df_merge.sort_values(by=['chr','site'])                
    
    #指定在一些列中将缺失值填充０,生成四条染色体文件
    df_merge=df_merge.fillna({'count1_x':0,'count2_x':0,'count1_y':0,'count2_y':0})
    df_merge=df_merge.fillna('NULL')
    
    #计算readcounts差异，生成汇总文件
    df_merge['readcounts_diff']=np.where(df_merge['count2_x']-df_merge['count2_y']>=0,(df_merge['count2_x']-df_merge['count2_y'])/df_merge['count2_x'],\
    (df_merge['count2_x']-df_merge['count2_y'])/df_merge['count2_y'])
    df_merge=df_merge.iloc[:,[0,1,4,5,9,10,12,3,8,6,11,2,7]]
    df_merge.to_csv('%s/%s_%s_alldp.xls'%(pwd,tag1,tag2),sep='\t',header=True,index=False)
    
    #统计，筛选，报告文件生成
    #统计行数,也可以用df1.shape[0]和df1.shape[1]
    df1_rownum=len(df1.index) 
    df2_rownum=df2.shape[0]
    df_merge_rownum=df_merge.shape[0]
    #统计unique_reads
    tag1_muniq=df_merge[df_merge['seq_y']=='NULL']['seq_y'].value_counts()
    tag2_muniq=df_merge[df_merge[u'seq_x']=='NULL']['seq_x'].value_counts()
    
    #报告文件生成
    f=open("%s/%s_%s_report.xls"%(pwd,tag1,tag2),'w')
    print("%s_reads\t%d"%(tag1,df1_rownum),"%s_reads\t%d"%(tag2,df2_rownum),\
    "merge_reads\t%d"%df_merge_rownum,"%s_muniq\t%d"%(tag1,tag1_muniq),\
    "%s_muniq\t%d"%(tag2,tag2_muniq),sep='\n',file=f)
    f.close() 
    
    #绘制散点图
    plt.plot(df_merge['readcounts_diff'],'|',fillstyle='none',markersize=5,color='#00aa00') 
    plt.ylim(-1.03,1.03)
    plt.xlabel('unique_read')
    plt.ylabel('readcounts_diff')
    plt.title('%s_reads vs %s_reads'%(tag1,tag2))
    plt.savefig('%s/%s_reads vs %s_reads.pdf'%(pwd,tag1,tag2),dpi=1080*720)

def libs_plus(filenames):
    filenames_list = filenames.split('/')
    tag,names = '',''
    for i in filenames_list:
        names = names + ' ' + i
        t = i.split('_')[0]
        tag = tag + t
    file_combine = tag + '.xls'
    os.system('cat %s > %s'%(names,file_combine))
    df = pd.read_table(file_combine,header=None)
    df1 = df.iloc[:,[0,1,2]]
    df2 = df1.groupby([0,1]).count()
    print("Uniq reads: %d"%len(df2))
    df2.to_csv('%s_combcount.xls'%tag,sep='\t',header=False)
if __name__ == '__main__':
    if len(sys.argv) < 2:
        print("Usage:Please input filenames of library1 and library2")
    elif len(sys.argv) == 2:
        filename1=sys.argv[1]
        filename2=sys.argv[2]
        libreadcounts1_vs_libreadcounts2(filename1,filename2)
        


#!/home/user/eib202epgs/bin/bin/python
# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------------#
# USAGE: ./jx                                                                        #
# AUTHOR: ZJX (Jim Chu), biojxz@gmail.com                                            #
# ORGANIZATION: ECUST                                                                #
# VERSION: 1.0                                                                       #
# CREATED: 2016年04月27日 15时20分01秒                                                 #
#------------------------------------------------------------------------------------#
import log
import yaml
import os,sys


#====================== <1:基于click技术的shell命令程序初始化> ======================#
import click 
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(version='1.0.0')
def binf():
    pass

#============================= <2:模块内部函数调用程序> =============================#
def procedure(module,procedure):
    global logger
    pwd1=os.getcwd()
    sys.path.append(pwd1)
    sys.path.append('/home/user/eib202epgs/zjx/python/jx-module')
    #logger = log.createCustomLogger('root')
    exec('import %s'%module)
    logger.info('Procedure is begining to run ...')    
    for i in procedure:
        fun = eval("%s.all_function['%s']"%(module,i))  #提取要执行的函数
        logger.info('Start ' + fun.split('(')[0] + ' ...')
        Return = eval(module+'.'+fun) 
        OI = fun.split('(')[0] + ' is over!'
        infor = OI if Return == None else "%s: %s \n%s"%(fun.split('(')[0],Return,OI)
        logger.info(infor)
    logger.info('Procedure is over!!')


#============================== <3: 子程序脚本分割线> ===============================#

#*********************** <3.1:Tnseq原始数据mapping和处理tntr> ***********************#
@binf.command(help='Tnseq site mapping and data treating.')
@click.option('--filename','-f',default='2.Tndata_raw/*',\
help='Please input Tnseq raw data filename.')
@click.option('--length','-l',default=80,\
help="Please input the length of reserved reads,default is 80")
@click.option('--transposon','-t',prompt='Transposon',\
help="Please input transposon abbreviated name,such as 'TcB'.")
@click.option('--pairedend','-pe',is_flag=True,\
help="If you want to analyze single end, please add '-npe'.")
@click.option('--procedure','-p',default='cbus',help='The selection of procedure:\
c:cutadapt;b:bowtie; u:unique;s:site_annot;r:cutreads. Default is cbus.')
def tntr(**kwargs):
    global filename,length,transposon,pe,manifest_data
    ##变量定义
    module = 'tnseqdt'
    filename = kwargs['filename']
    length = kwargs['length']
    transposon = manifest_data['transposon'][kwargs['transposon']]
    pe = kwargs['pairedend']
    dirname = filename.split('/')[0]
    filename = filename.split('/')[-1]
    ##调用全部或部分tnseqdt模块内部的类或函数## 
    if dirname == '2.Tndata_raw':
        procedure(module,kwargs['procedure'])
    else:
        print("Please choose '2.Tndata_raw' folder with '-f' option")
        sys.exit()

#*************************** <3.2:数据可视化命令visualization> **************************#
@binf.command(help='Visualization of Tnseq data.')
@click.option('--dirname','-d',prompt='Dirname',help='Please input objective dirname.')
@click.option('--procedure','-p', default='ws',help='The selection of procedure. \
w:weblogo, s:site_distribution, d:DNAplotter. Default is all.')
def visualization(**kwargs):
    global dirname
    module = 'visualization'
    dirname = kwargs['dirname']
    procedure(module,kwargs['procedure'])

#*************************** <3.3:多个测序库比较命令libvs> **************************#
@binf.command(help='Multi-seq-libraries compare.')
@click.option('--filename1','-f1',help='Please input filename of library1.')
@click.option('--filename2','-f2',help='Please input filename of library2.')
@click.option('--filenames','-fs',help='Please input filenames of librarys.')
@click.option('--procedure','-p', default='l',help='The selection of procedure. \
l:libreadcounts1_vs_libreadcounts2. Default is blastn.')
def libvs(**kwargs):
    global filename1,filename2,filenames
    module = 'libreadcounts1_vs_libreadcounts2'
    filename1 = kwargs['filename1']
    filename2 = kwargs['filename2']
    filenames = kwargs['filenames']
    procedure(module,kwargs['procedure'])

#**************** <3.4:测序序列、引物序列、长序列等序列比对命令blast> ***************#
@binf.command(help='Blast of sequnce and primer seq.')
@click.option('--database','-db',default='',help="Database_name.")
@click.option('--subject','-sj',default='',help='Seq file or folder contain seq.')
@click.option('--query','-q',prompt='query seq',help='Query sequences such as primer.')
@click.option('--task','-t',default='blastn-short',help="Permissible values: 'blastn',\
'blastn-short','dc-megablast','megablast','rmblastn';Default = 'blastn-short'.")
@click.option('--outfmt','-fmt',default=6,help="Formatting options, default is 6")
@click.option('--word_size','-w',default=14,help="Word size for wordfinder algorithm \
(length of best perfect match), default is 16.")
@click.option('--procedure','-p', default='b',help='The selection of procedure. \
b:blastn,i:internal_seq_extract_and_blastn. Default is all.')
def blast(**kwargs):
    import re
    global tag,result_folder,seq_folder,subtype,dbseq,seq,task,outfmt,word_size
    module = 'seqblast'
    proc = kwargs['procedure']
    if kwargs['database']:
        subtype,dbseq = 'db',kwargs['database']
        tag1 = re.split('_|\.',kwargs['database'])[0]
    elif kwargs['subject']:
        subtype,dbseq = 'subject',kwargs['subject']
        tag1 = re.split('_|\.',kwargs['subject'])[0]
        if os.path.isdir(kwargs['subject']):
            proc,seq_folder = 'e' + proc,kwargs['subject']
    else:
        print "Usage: please input subject or database name."
    tag2 = kwargs['query'].split('/')[-1].split('.')[0]
    result_folder = tag2 + '-' + tag1 + '_result'
    tag = tag2 + '-' + tag1
    if kwargs['subject']:
        dbseq = '%s/%s.seq'%(result_folder,tag)
    seq,task = kwargs['query'],kwargs['task']
    outfmt,word_size = kwargs['outfmt'],kwargs['word_size']
    procedure(module,proc)

#*************************** <3.5:必须基因分析esanalysis> **************************#
@binf.command(help='Essential analysis of Ppgene.')
@click.option('--filename','-f',prompt='filename',help='Please input objective filename.')
@click.option('--method','-m',default=1,help='Please input analysis method.Method 1 means \
retain repeated insertion reads, and 2 meads drop repeats. Default is 1.')
@click.option('--all_insert','-a',default=0,help='Total insertion number. Default is CDS \
total insertion number.')
@click.option('--estypefile','-ef',default='Sc.txt',help='Please input estypefile.')
@click.option('--procedure','-p', default='ge',help='The selection of procedure. \
g:gene_insert_density, e:es_mapes. Default is all.')
def esanalysis(**kwargs):
    global filename,method,all_insert,estypefile
    module = 'essential_assay'
    filename = kwargs['filename']
    method = kwargs['method']
    all_insert = kwargs['all_insert']
    estypefile = kwargs['estypefile']
    procedure(module,kwargs['procedure'])
#================================== <4:主程序运行> ==================================#                    
if __name__ == '__main__':
    logger = log.createCustomLogger('root')
    #logger.info('Loading manifest...')
    manifest_file = open('manifest.yaml')
    manifest_data = yaml.load(manifest_file)
    binf()










#!usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function
import tnseq_rawdata_treatng as tntr
import os,sys
import re
#import multiprocessing

##列出所有函数
all_function = {'r':'cutreads(filename,length)','c':'cutadapt(filename,transposon,pe)','b':'bowtie()',\
'u':'unique(transposon)','s':'site_annot(filename)','w':'weblogo(filename)','i':'gene_insert(filename)',\
'm':'mapES(filename)'}

filename_cr = ''
tag = ''
tag_cutadapt = ''
tag_bowtie = ''

def cutreads(filename,length):
    global filename_cr
    if filename[-3:] == '.gz':
        os.system('gunzip 2.Tndata_raw/%s'%filename)
        filename=filename[:-3]
    tntr.cutreads(filename,length)
    filename_cr = filename + '_cr'

def cutadapt(filename,transposon,pe):
    global tag,tag_cutadapt
    if filename_cr:
        filename = filename_cr
    tag=re.split('\.',filename)[0]
    tag_cutadapt=tag+'_t_l'
    if filename[-3:] == '.gz':
        os.system('gunzip 2.Tndata_raw/%s'%filename)
        filename=filename[:-3]
    if tag_cutadapt not in os.listdir('3.Cutadapt_trimmed'):
        tntr.cutadapt(filename,transposon,tag,pe)

def bowtie():
    global tag_bowtie
    tag_bowtie=tag+'_bowtie.txt'
    if tag_bowtie not in os.listdir('4.Bowtie_data_dir'):
        tntr.bowtie(tag_cutadapt,tag_bowtie)

def unique(transposon):
    if not os.path.exists(tag):
        os.system('mkdir %s'%tag)
    if tag+'_chr4_alldp.xls' not in os.listdir(tag):
        tntr.unique('4.Bowtie_data_dir/'+tag_bowtie,tag,transposon)

def site_annot(filename):
    global tag
    tag = re.split('\.',filename)[0]
    num_all,num_CDS = 0,0
    num_dict = {}
    for j in range(1,5):
        j='chr'+str(j)
        num_dict = tntr.site_annot(tag,j)
        num_all = num_all + num_dict['All_count']
        num_CDS = num_CDS + num_dict['CDS_count']
    print('-----The number of All insertion reads is %d-----'%num_all)
    print('-----The number of CDS insertion reads is %d-----\n'%num_CDS)
    os.system('cat %s/*chr*_CDS.xls > %s/%s_CDS.xls'%(tag,tag,tag))
    os.system('cat %s/*chr*_IGS.xls > %s/%s_IGS.xls'%(tag,tag,tag))

def gene_insert(filename):
    dirname = re.split('\.',filename)[0]
    for i in range(1,5):
        chro = 'chr' + str(i)
        tntr.gene_insert(dirname,chro)
    os.system('cat %s/*chr*_CDS_insert.xls > %s/%s_CDS_insert.xls'%(dirname,dirname,dirname))

def mapES(filename):
    dirname = re.split('\.',filename)[0]
    tntr.mapes('ESpredictbySL',dirname)
"""
def tnseqmaptr_all(transposon): 
    def tsmt(filename,transposon):  
        Tnseqmaptr(filename,transposon).cutadapt()
        Tnseqmaptr(filename,transposon).bowtie()
        Tnseqmaptr(filename,transposon).unique()
        Tnseqmaptr(filename,transposon).site_annot()
        weblogo(filename)
    #pool=multiprocessing.Pool(4)
    for i in os.listdir('2.Tndata_raw'):
        multiprocessing.Process(target=tsmt,args=(i,transposon,)).start()
        #p.join()
"""

if __name__=='__main__':
    transposon=sys.argv[1]
    """for i in range(1,5):
        tn.Weblogo('Thm2','chr'+str(i))
    except Exception,ex: 
        os.system("echo '%s:%s' | mail -s 'Infor' \
        347909696@qq.com,biojxz@163.com"%(Exception,ex))  """       
    
    
    


            




from __future__ import print_function
from pyfaidx import Fasta
import pandas as pd
import numpy as np
from Bio.Seq import Seq
import os

def dnaplotter_artemis(tag):
    #df = pd.read_table('T16728-12b_combined_R1/T16728-12b_combined_R1_alldp.xls',usecols=[0,1,2],names=['chro','site','direct'])
    df = pd.read_table('%s/%s_alldp.xls'%(tag,tag),usecols=[0,1,2],names=['chro','site','direct'])
    CL={'chr1':2798491,'chr2':2394163,"chr3":2245428,"chr4":1778296}
    def chro_proc(chro):
        df_chro = df[df['chro']==chro]
        df_chro = df_chro.groupby(['site','direct'],as_index=False).count()
        #df_chro['plotter'] = np.where(df_chro['direct']=='+',df_chro['chro'],-1*df_chro['chro'])
        df_chro['F'] = np.where(df_chro['direct']=='+',df_chro['chro'],0)
        df_chro['R'] = np.where(df_chro['direct']=='-',-1*df_chro['chro'],0)
        df_coordinate = np.arange(CL[chro])
        df_coordinate = pd.DataFrame(df_coordinate)
        df_map = pd.merge(df_coordinate,df_chro,left_on=0,right_on='site',how='left')
        df_map = df_map.fillna(0)
        df_map_mod = df_map[df_map['site']!=0]
        df_map.to_csv('%s/%s_%s_plus.txt'%(tag,tag,chro),sep='\t',index=False,columns=['F'],header=False)
        df_map.to_csv('%s/%s_%s_minus.txt'%(tag,tag,chro),sep='\t',index=False,columns=['R'],header=False)
        df_map_mod.to_csv('%s/%s_%s_artemis.txt'%(tag,tag,chro),sep='\t',index=False,columns=['site','F','R'])
    for i in range(1,5):
        chro = 'chr' + str(i)
        chro_proc(chro)

def DNAplotter_reads(tag,chromosome,ic='uniq'):
    f1=open('%s/%s_%s_alldp.xls'%(tag,tag,chromosome),"r")
    forward,reverse = {},{}
    CL={'chr1':2798491,'chr2':2394163,"chr3":2245428,"chr4":1778296}
    for i in f1:
        i=i.split()
        n=int(i[1])
        if ic=='uniq':
            if i[2]=='+':
                forward[n]=1
            else:
                reverse[n]=-1
        else:
            if i[2]=='+':
                forward[n]=i[5]
            else:
                reverse[n]=-int(i[5])            
    def Data_transform(d,o,tag,chromosome):
        f2=open('%s/%s_%s_%s.txt'%(tag,tag,chromosome,o),'w')
        chr_length=CL[chromosome]
        for i in range(1,chr_length+1):
            if i in d.keys():
                print(d[i],file=f2)
            else:
                f2.write('0\n')
        f2.close()
    Data_transform(forward,'plus',tag,chromosome)
    Data_transform(reverse,'minus',tag,chromosome)
    f1.close()

def Site_distribution(argv1,argv2):
    wm='w' if argv2=='chr1' else 'a'
    f=open('%s/%s_sd.xls'%(argv1,argv1),wm)
    f1=open('%s/%s_%s_IGS.xls'%(argv1,argv1,argv2))
    for i in f1:
        i=i.split()
        i[1],i[4],i[5]=float(i[1]),float(i[4]),float(i[5])
        if i[1] >= (i[4]+i[5])/2:
            sd=(i[1]-i[5])/1000
        else:
            sd = 1+(i[1]-i[4])/1000
        print(argv2,sd,sep='\t',file=f)
    f.close()
    f1.close()
    f=open('%s/%s_sd.xls'%(argv1,argv1),'a')
    f2=open('%s/%s_%s_CDS.xls'%(argv1,argv1,argv2))
    for i in f2:
        i=i.split()
        i[1],i[7],i[12]=float(i[1]),float(i[7]),float(i[12])
        sd=(i[1]-i[7])/i[12]
        print(argv2,sd,sep='\t',file=f)
    f.close()

def R_sd(tag):
    f=open('%s/%s_sd.R'%(tag,tag),'w')
    
    Rscript="""library("ggplot2")
    fa<-read.table("%s/%s_sd.xls")
    colnames(fa) <-c("Chromosome","Insertion.site")
    fb<-fa[fa[,2]>-3,]
    fb<-fb[fb[,2]<3,]
    pdf("%s/%s_sd.pdf")
    qplot(Insertion.site,data=fb,geom = "density",colour=Chromosome)
    dev.off()
    """%(tag,tag,tag,tag)
    
    print(Rscript,file=f)
    f.close()
    os.system('R2 CMD BATCH %s/%s_sd.R'%(tag,tag))

#def Pichia_TA_weblogo():


def Weblogo(tag):
    f1=open('%s/%s_alldp.xls'%(tag,tag),"r")
    f2=open('%s/%s_base.txt'%(tag,tag),'w')
    #f3=open('%s/%s_basecount.txt'%(tag,tag),"w")
    pichia_genome_path = '/home/user/eib202epgs/zjx/ref/pichia/genome.fa'
    pichia_genome = Fasta(pichia_genome_path)
    seq,count='',0
    D={}
    D2 = {'chr1':'NC_012963.1','chr2':'NC_012964.1','chr3':'NC_012965.1','chr4':'NC_012966.1'}
    """for i in range(1,19):
        D['s'+str(i)+'A']=0  #some site for A
        D['s'+str(i)+'C']=0
        D['s'+str(i)+'G']=0
        D['s'+str(i)+'T']=0"""
    for j in f1:
        count=count+1
        j=j.split()
        chromosome = D2[j[0]]
        site = int(j[1])
        Targetseq = pichia_genome[chromosome][site-9:site+9]
        Targetseq = str(Targetseq)
        if j[2] == '-':
            Targetseq = str(Seq(Targetseq).reverse_complement())
        print('>%s_%d\n%s'%(chromosome,count,Targetseq),file=f2)
        """for k in range(1,18):
            D['s'+str(k)+seq[k-1]]+=1
    for i in range(1,18):
        for j in ['A','C','G','T']:
            print('s'+str(i),j,D['s'+str(i)+j],sep='\t',file=f3)"""
    f1.close()
    f2.close()
    #f3.close()
    os.system("weblogo -c classic --format pdf < %s/%s_base.txt > \
    %s/%s_base.pdf"%(tag,tag,tag,tag))
    


"""
Build tnseq_yeast.
"""

from setuptools import setup, Extension

requirements = [line.strip() for line in open('requirements.txt')]

setup(
	name = 'tnseq_yeast',
	version = '0.2', #versioneer.get_version(),
	author = 'Jim Chu',
	author_email = 'biojxz@163.com',
	url = 'https://tnseq_yeast.readthedocs.io/',
	description = 'trim adapters from high-throughput sequencing reads',
	license = 'MIT',
	#cmdclass = cmdclass,
	#ext_modules = extensions,
	packages = ['tnseq_yeast', 'tnseq_yeast.scripts'],
	scripts = ['bin/tnseqy'],
	install_requires=requirements,
	classifiers = [
		"Development Status :: 5 - Production/Stable",
		"Environment :: Console",
		"Intended Audience :: Science/Research",
		"License :: OSI Approved :: MIT License",
		"Natural Language :: English",
		"Programming Language :: Cython",
		"Programming Language :: Python :: 2.7",
		"Programming Language :: Python :: 3",
		"Topic :: Scientific/Engineering :: Bio-Informatics"
	]
)



# -*- coding: UTF-8 -*-

"""
1，更新毕赤酵母注释信息
2，分析"TA","TTAA"等bases分布
3，目前bases两位点(用于bedtools intersect分析的start && end)暂时为bases第一个碱基分别以染色体起始位点为0和1的对于位点
"""
from __future__ import print_function
import os,re
from pyfaidx import Fasta

def bases_distribution(strain_name,bases):
    ref_path="/home/user/eib202epgs/zjx/ref"
    strain_ref_path="%s/%s"%(ref_path,strain_name)
    bases_distribution_file="%s_%s_distribution"%(strain_name,bases)
    FIN=open('%s.bed'%bases_distribution_file,'w')
    genome_path="%s/genome.fa"%strain_ref_path
    genome=Fasta(genome_path)
    for i in genome.keys():
        seq=str(genome[i][:]).upper()
        for j in re.finditer(bases,seq):
            print(i,j.span()[0],j.span()[0]+1,j.group(),sep='\t',file=FIN)
    FIN.close()
    CMD_intersect="bedtools intersect -a %s/genome.gff -b %s.bed -wa -wb > %s.intersect.xls"\
    %(strain_ref_path,bases_distribution_file,bases_distribution_file)
    os.system(CMD_intersect)
    return CMD_intersect	
    
if __name__ == '__main__':
    import log
    import sys
    logger = log.createCustomLogger('root')
    strain_name=sys.argv[1]
    bases=sys.argv[2]
    logger.info("bases_distribution is begining...")
    CMD_intersect=bases_distribution(strain_name,bases)
    logger.info("The intersect command is '%s'"%CMD_intersect)
    logger.info("bases_distribution is over!")



from __future__ import print_function
from Bio import Entrez

#从DEG10必须基因数据库中提取S.pombe必须基因的gi
def essential_gi():
    f = open('Sp_es_gi.txt')
    gi_list = []
    for i in f:
        gi_list.append(i.split()[2])
    f.close()
    return gi_list

def extract_NP(id_list):
    f = open('Sp_es_NP.txt','w')
    #eg. id_list = ["429241493","19113749"]
    handle = Entrez.esummary(db="protein", id=",".join(id_list))
    record = Entrez.read(handle)
    NP_list = []
    for key in record:
        NP = record[key]['Extra'].split('|')[3]
        print(NP,file=f)
    f.close()
    return NP_list

if __name__=='__main__':
    id_list = essential_gi()
    extract_NP(id_list)